package gov.anl.aps.dm.common.objects;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import gov.anl.aps.dm.common.exceptions.DmException;
import java.lang.reflect.Type;
import java.util.LinkedList;
import java.util.List;
import org.apache.log4j.Logger;

/**
 * DM object factory class.
 */
public class DmObjectFactory {

    private static final Logger logger = Logger.getLogger(DmObjectFactory.class.getName());
    private static final Gson gson = new GsonBuilder().create();

    /**
     * Create object from JSON string.
     *
     * @param <T> template class
     * @param jsonString JSON string
     * @param objectClass object class
     * @return generated object
     */
    public static <T extends Object> T createObject(String jsonString, Class<T> objectClass) {
        logger.debug("Converting JSON string to object " + objectClass + ": " + jsonString);
        T object = gson.fromJson(jsonString, objectClass);
        return object;
    }

    /**
     * Create DM object from JSON string.
     *
     * @param <T> template class
     * @param jsonString JSON string
     * @param dmClass DM object class
     * @return generated DM object
     * @throws DmException in case of any errors
     */
    public static <T extends DmObject> T createDmObject(String jsonString, Class<T> dmClass) throws DmException {
        logger.debug("Converting JSON string to DM object " + dmClass + ": " + jsonString);
        T dmObject = gson.fromJson(jsonString, dmClass);
        dmObject.decode();
        return dmObject;
    }

    /**
     * Create list of DM objects from JSON string.
     *
     * @param <T> template class
     * @param jsonString DM string
     * @return generated list of DM objects
     */
    public static <T extends DmObject> List<T> createDmObjectList(String jsonString) {
        // This method does not appear to work as template, so we have
        // to write specific methods for each object type.
        logger.debug("Converting JSON string to dm object list: " + jsonString);
        Type dmType = new TypeToken<LinkedList<T>>() {
        }.getType();
        List<T> dmObjectList = gson.fromJson(jsonString, dmType);
        return dmObjectList;
    }

    /**
     * Create list of string objects from JSON string.
     *
     * @param jsonString JSON string
     * @return generated list of string objects
     */
    public static List<String> createStringObjectList(String jsonString) {
        logger.debug("Converting JSON string to string object list: " + jsonString);
        Type dmType = new TypeToken<LinkedList<String>>() {
        }.getType();
        List<String> dmObjectList = gson.fromJson(jsonString, dmType);
        return dmObjectList;
    }

}
