package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.common.exceptions.DmException;
import gov.anl.aps.dm.common.exceptions.InvalidRequest;
import gov.anl.aps.dm.common.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.model.beans.ExperimentStationDbFacade;
import gov.anl.aps.dm.portal.model.beans.SystemRoleTypeDbFacade;
import gov.anl.aps.dm.portal.model.beans.UserSystemRoleDbFacade;
import gov.anl.aps.dm.portal.model.entities.ExperimentStation;
import gov.anl.aps.dm.portal.model.entities.SystemRoleType;
import gov.anl.aps.dm.portal.model.entities.UserInfo;
import gov.anl.aps.dm.portal.model.entities.UserSystemRole;
import gov.anl.aps.dm.portal.model.entities.UserSystemRolePK;
import gov.anl.aps.dm.portal.utilities.AuthorizationUtility;
import gov.anl.aps.dm.portal.utilities.SessionUtility;
import java.util.ArrayList;
import java.util.List;
import javax.ejb.EJB;
import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import org.apache.log4j.Logger;

@Named("experimentStationController")
@SessionScoped
public class ExperimentStationController extends DmEntityController<ExperimentStation, ExperimentStationDbFacade> {

    private static final Logger logger = Logger.getLogger(ExperimentStationController.class.getName());

    @EJB
    private ExperimentStationDbFacade experimentStationDbFacade;
    @EJB
    private SystemRoleTypeDbFacade systemRoleTypeDbFacade;
    @EJB
    private UserSystemRoleDbFacade userSystemRoleDbFacade;
     
    UserSystemRole currentUserSystemRole;

    public ExperimentStationController() {
    }

    @Override
    protected ExperimentStationDbFacade getFacade() {
        return experimentStationDbFacade;
    }

    @Override
    protected ExperimentStation createEntityInstance() {
        return new ExperimentStation();
    }

    @Override
    public String getEntityTypeName() {
        return "experimentStation";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getName();
        }
        return "";
    }

    @Override
    public ExperimentStation findById(Integer id) {
        return experimentStationDbFacade.findById(id);
    }

    @Override
    public void prepareEntityInsert(ExperimentStation experimentStation) throws DmException {
        if ((experimentStation.getName() == null) || (experimentStation.getName().length() == 0)) {
            throw new InvalidRequest("Experiment station name is missing.");
        }
        ExperimentStation existingExperimentStation = experimentStationDbFacade.findByName(experimentStation.getName());
        if (existingExperimentStation != null) {
            throw new ObjectAlreadyExists("Experiment station " + experimentStation.getName() + " already exists.");
        }
        logger.debug("Inserting new experiment station " + experimentStation.getName());
    }

    @Override
    public void prepareEntityUpdate(ExperimentStation experimentStation) throws DmException {
        if ((experimentStation.getName() == null) || (experimentStation.getName().length() == 0)) {
            throw new InvalidRequest("Experiment station name is missing.");
        }
        logger.debug("Updating experiment station " + experimentStation.getName());
    }

    @Override
    protected String getObjectAlreadyExistMessage(ExperimentStation experimentStation) {
        if (experimentStation == null) {
            return null;
        }
        return "Experiment station " + experimentStation.getName() + " already exists.";
    }

    public List<ExperimentStation> getAvailableExperimentStations() {
        UserInfo user = (UserInfo) SessionUtility.getUser();
        if (AuthorizationUtility.isAdministrator(user)) {
            return getFacade().findAll();
        }
        ArrayList<ExperimentStation> availableStations = new ArrayList<>();
        user.getUserSystemRoleList().stream().map((userSystemRole) -> userSystemRole.getExperimentStation()).filter((experimentStation) -> (experimentStation != null)).forEachOrdered((experimentStation) -> {
            availableStations.add(experimentStation);
        });
        return availableStations;
    }

    public UserSystemRole getCurrentUserSystemRole() {
        if (currentUserSystemRole == null) {
            currentUserSystemRole = new UserSystemRole();
        }
        return currentUserSystemRole;
    }

    public void setCurrentUserSystemRole(UserSystemRole currentUserSystemRole) {
        this.currentUserSystemRole = currentUserSystemRole;
    }

    public void prepareAddManager(ExperimentStation experimentStation) {
        UserSystemRole userSystemRole = getCurrentUserSystemRole();
        userSystemRole.setSystemRoleType(systemRoleTypeDbFacade.findManagerRoleType());
        userSystemRole.setExperimentStation(experimentStation);
    }

    public void addManager() {
        UserInfo user = currentUserSystemRole.getUserInfo();
        ExperimentStation experimentStation = currentUserSystemRole.getExperimentStation();
        SystemRoleType roleType = currentUserSystemRole.getSystemRoleType();
        UserSystemRolePK userSystemRolePK = new UserSystemRolePK();
        userSystemRolePK.setUserId(user.getId());
        userSystemRolePK.setRoleTypeId(roleType.getId());
        currentUserSystemRole.setUserSystemRolePK(userSystemRolePK);

        UserSystemRole existingRole = userSystemRoleDbFacade.findByUserAndRoleTypeAndExperimentStation(user.getId(),
                roleType.getId(), experimentStation.getId());
        if (existingRole != null) {
            SessionUtility.addErrorMessage("Error", "Could not add station manager: User "
                    + user.getUsername()
                    + " is already as manager of "
                    + experimentStation.getName() + " station.");
            return;
        }
        List<UserSystemRole> userSystemRoleList = experimentStation.getUserSystemRoleList();
        userSystemRoleList.add(0, currentUserSystemRole);
        update();
        currentUserSystemRole = null;
    }

    public void deleteManager(UserSystemRole userSystemRole) {
        ExperimentStation experimentStation = getCurrent();
        userSystemRoleDbFacade.remove(userSystemRole);
        List<UserSystemRole> userSystemRoleList = experimentStation.getUserSystemRoleList();
        userSystemRoleList.remove(userSystemRole);
        updateOnRemoval();
    }

    @FacesConverter(forClass = ExperimentStation.class)
    public static class ExperimentStationControllerConverter implements Converter {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            ExperimentStationController controller = (ExperimentStationController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "experimentStationController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof ExperimentStation) {
                ExperimentStation o = (ExperimentStation) object;
                return getStringKey(o.getId());
            } else {
                throw new IllegalArgumentException("Object " + object + " is of type " + object.getClass().getName() + "; expected type: " + ExperimentStation.class.getName());
            }
        }

    }

}
